package com.ruoyi.system.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson2.JSON;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.core.domain.entity.AsUser;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.uuid.IdUtils;
import com.ruoyi.system.domain.EtWithdraw;
import com.ruoyi.system.mapper.AsUserMapper;
import com.ruoyi.system.mapper.EtWithdrawMapper;
import com.ruoyi.system.mapper.SysUserMapper;
import com.ruoyi.system.service.IEtWithdrawService;
import com.ruoyi.system.service.ISysDeptService;
import com.ruoyi.system.service.IWxPayService;
import com.wechat.pay.java.service.transferbatch.model.TransferDetailInput;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;


/**
 * 提现记录Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-06-27
 */
@Slf4j
@Service
public class EtWithdrawServiceImpl implements IEtWithdrawService
{
    @Resource
    private EtWithdrawMapper etWithdrawMapper;

    @Resource
    private AsUserMapper asUserMapper;

    @Resource
    private SysUserMapper userMapper;

    @Autowired
    private IWxPayService wxPayService;

    @Autowired
    private ISysDeptService deptService;

    /**
     * 查询提现记录
     *
     * @param id 提现记录主键
     * @return 提现记录
     */
    @Override
    public EtWithdraw selectEtWithdrawById(Long id)
    {
        return etWithdrawMapper.selectEtWithdrawById(id);
    }

    /**
     * 查询提现记录列表
     *
     * @param etWithdraw 提现记录
     * @return 提现记录
     */
    @Override
    public List<EtWithdraw> selectEtWithdrawList(EtWithdraw etWithdraw)
    {
        return etWithdrawMapper.selectEtWithdrawList(etWithdraw);
    }

    /**
     * 新增提现记录
     *
     * @param etWithdraw 提现记录
     * @return 结果
     */
    @Override
    public int insertEtWithdraw(EtWithdraw etWithdraw)
    {
        SysUser user = SecurityUtils.getLoginUser().getUser();
        user = userMapper.selectUserById(user.getUserId());
        etWithdraw.setCreateTime(DateUtils.getNowDate());
        etWithdraw.setWithdrawNo(IdUtils.getOrderNo("tx"));
        etWithdraw.setApplicant(user.getUserName());
        etWithdraw.setDeptId(user.getDeptId());
        if(ObjectUtil.isNull(user.getAppUserId())){
            throw new ServiceException("请先绑定微信");
        }
        AsUser asUser = asUserMapper.selectUserById(user.getAppUserId());
        log.info("获取到app用户:{}", JSON.toJSON(asUser));
        etWithdraw.setWxopenid(asUser.getWxopenid());
        etWithdraw.setStatus(ServiceConstants.WITHDRAW_STATUS_APPLY);
        return etWithdrawMapper.insertEtWithdraw(etWithdraw);
    }

    /**
     * 修改提现记录
     *
     * @param etWithdraw 提现记录
     * @return 结果
     */
    @Override
    public int updateEtWithdraw(EtWithdraw etWithdraw)
    {
        //如果提现是通过，则发起商户转账到零钱
        if(etWithdraw.getStatus().equals(ServiceConstants.WITHDRAW_STATUS_PASS)){
            SysDept sysDept = deptService.selectDeptById(etWithdraw.getDeptId());
            String batchName= sysDept.getDeptName()+"发起提现";
            List<TransferDetailInput> transferDetailInputs = new ArrayList<>();
            TransferDetailInput transferDetailInput = new TransferDetailInput();
            transferDetailInput.setOutDetailNo(IdUtils.getOrderNo("txd"));//明细订单号
            transferDetailInput.setTransferAmount(etWithdraw.getAmount().multiply(new BigDecimal(100)).longValue());
            transferDetailInput.setTransferRemark("创特共享电动车提现");
            transferDetailInput.setOpenid(etWithdraw.getWxopenid());
            transferDetailInputs.add(transferDetailInput);
            log.info("【审核通过】发起转账到零钱:{}",JSON.toJSONString(transferDetailInputs));
            wxPayService.transfer(sysDept,etWithdraw.getWithdrawNo(),batchName,batchName,etWithdraw.getAmount(),1,transferDetailInputs);
        }
        return etWithdrawMapper.updateEtWithdraw(etWithdraw);
    }

    /**
     * 批量删除提现记录
     *
     * @param ids 需要删除的提现记录主键
     * @return 结果
     */
    @Override
    public int deleteEtWithdrawByIds(Long[] ids)
    {
        return etWithdrawMapper.deleteEtWithdrawByIds(ids);
    }

    /**
     * 删除提现记录信息
     *
     * @param id 提现记录主键
     * @return 结果
     */
    @Override
    public int deleteEtWithdrawById(Long id)
    {
        return etWithdrawMapper.deleteEtWithdrawById(id);
    }
}
